/*
 * ScatterPlot.java
 *
 * Created on September 8, 2005, 10:52 PM
 * Modified on January 29, 2007 (Mark Fitzgerald)
 */

package org.neptuneinc.cadstat.plots;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultComboBoxModel;
import javax.swing.SpinnerNumberModel;

import org.neptuneinc.cadstat.prefs.PreferencesManager;
import org.neptuneinc.cadstat.ui.DataPlotDialog;
import org.neptuneinc.cadstat.utils.GUIUtils;
import org.neptuneinc.cadstat.utils.RUtils;
import org.rosuda.JGR.JGR;

/**
 *
 * @author  Pasha Minallah
 */
public class ScatterPlot extends DataPlotDialog
{
  /** Creates new form ScatterPlot */
  public ScatterPlot()
  {
    super();
  }

  /** Perform custom initialization. */
  @Override
  protected void initCustom()
  {
    this.initComponents();
    
    this.getDatasetPane().getDatasetComboBox().addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        datasetComboBoxActionPerformed(e);
      }});
    this.getDatasetPane().refreshDatasetComboBox();
    
    this.getPlotPane().add(plotPane, BorderLayout.CENTER);
  }

  private void datasetComboBoxActionPerformed(ActionEvent e)
  {
    this.refreshVariableList();
    this.refreshSubmitButtonValidity();
  }
  
  public void refreshVariableListValidity()
  {
    xComboBox.setEnabled(this.getDatasetPane().getDatasetComboBox().getItemCount() > 0 && xComboBox.getItemCount() > 0);
    yComboBox.setEnabled(this.getDatasetPane().getDatasetComboBox().getItemCount() > 0 && yComboBox.getItemCount() > 0);
  }
  
  public void refreshVariableList()
  {
    if (this.getDatasetPane().getDatasetComboBox().getItemCount() > 0)
    {
      Vector nonFactors = RUtils.nonFactors(this.getDatasetPane().getSelectedDataset());
      xComboBox.setModel(new DefaultComboBoxModel(nonFactors));
      yComboBox.setModel(new DefaultComboBoxModel(nonFactors));
      
      if (xComboBox.getItemCount() > 0)
      {
        xComboBox.setSelectedIndex(0);
      }
      
      if (yComboBox.getItemCount() > 0)
      {
        yComboBox.setSelectedIndex(0);
      }
    }
    
    else
    {
      xComboBox.removeAllItems();
      yComboBox.removeAllItems();
    }
    
    this.refreshVariableListValidity();
    this.refreshSubmitButtonValidity();
  }
  
  public void refreshSubmitButtonValidity()
  {
    this.getSubmitButton().setEnabled(this.getDatasetPane().getDatasetComboBox().getItemCount() > 0 &&
      xComboBox.getItemCount() > 0 && yComboBox.getItemCount() > 0);
  }
  public void refreshConfPredInt()
  {
      confIntCheck.setEnabled(regLineCheck.isSelected()||loessLineCheck.isSelected());
      predIntCheck.setEnabled(regLineCheck.isSelected()||loessLineCheck.isSelected());
      if(!(regLineCheck.isSelected()||loessLineCheck.isSelected())){
        confIntCheck.setSelected(false);
        predIntCheck.setSelected(false);
      }
      confLabel.setEnabled(regLineCheck.isSelected()||loessLineCheck.isSelected());
      confLevelSpinner.setEnabled(regLineCheck.isSelected()||loessLineCheck.isSelected());
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        plotPane = new javax.swing.JPanel();
        varPanel = new javax.swing.JPanel();
        xLabel = new javax.swing.JLabel();
        xComboBox = new javax.swing.JComboBox();
        yLabel = new javax.swing.JLabel();
        yComboBox = new javax.swing.JComboBox();
        plotOptionsPane = new javax.swing.JPanel();
        regLineCheck = new javax.swing.JCheckBox();
        loessLineCheck = new javax.swing.JCheckBox();
        predIntCheck = new javax.swing.JCheckBox();
        confLabel = new javax.swing.JLabel();
        confLevelSpinner = new javax.swing.JSpinner();
        confIntCheck = new javax.swing.JCheckBox();
        plotLabelsPane = new javax.swing.JPanel();
        plotTitleLabel = new javax.swing.JLabel();
        plotTitle = new javax.swing.JTextField();
        xAxisLabel = new javax.swing.JLabel();
        xAxis = new javax.swing.JTextField();
        yAxisLabel = new javax.swing.JLabel();
        yAxis = new javax.swing.JTextField();

        varPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Variables"));

        xLabel.setText("X");

        yLabel.setText("Y");

        org.jdesktop.layout.GroupLayout varPanelLayout = new org.jdesktop.layout.GroupLayout(varPanel);
        varPanel.setLayout(varPanelLayout);
        varPanelLayout.setHorizontalGroup(
            varPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(varPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(varPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(varPanelLayout.createSequentialGroup()
                        .add(xLabel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(xComboBox, 0, 166, Short.MAX_VALUE))
                    .add(varPanelLayout.createSequentialGroup()
                        .add(yLabel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(yComboBox, 0, 166, Short.MAX_VALUE)))
                .addContainerGap())
        );
        varPanelLayout.setVerticalGroup(
            varPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(varPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(varPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(xLabel)
                    .add(xComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(varPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(yLabel)
                    .add(yComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(39, Short.MAX_VALUE))
        );

        plotOptionsPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Plot Options"));

        regLineCheck.setText("Linear Regression Line");
        regLineCheck.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                regLineCheckActionPerformed(evt);
            }
        });

        loessLineCheck.setText("Loess Smooth");
        loessLineCheck.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loessLineCheckActionPerformed(evt);
            }
        });

        predIntCheck.setText("Add Prediction Intervals");
        predIntCheck.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        predIntCheck.setEnabled(false);
        predIntCheck.setMargin(new java.awt.Insets(0, 0, 0, 0));

        confLabel.setText("Confidence / Prediction Level");
        confLabel.setEnabled(false);

        confLevelSpinner.setModel(new javax.swing.SpinnerNumberModel(0.95d, 0.5d, 0.99d, 0.01d));
        confLevelSpinner.setEnabled(false);

        confIntCheck.setText("Add Confidence Intervals");
        confIntCheck.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        confIntCheck.setEnabled(false);

        org.jdesktop.layout.GroupLayout plotOptionsPaneLayout = new org.jdesktop.layout.GroupLayout(plotOptionsPane);
        plotOptionsPane.setLayout(plotOptionsPaneLayout);
        plotOptionsPaneLayout.setHorizontalGroup(
            plotOptionsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, plotOptionsPaneLayout.createSequentialGroup()
                .addContainerGap()
                .add(plotOptionsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(regLineCheck)
                    .add(loessLineCheck))
                .add(30, 30, 30)
                .add(plotOptionsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(plotOptionsPaneLayout.createSequentialGroup()
                        .add(confIntCheck)
                        .addContainerGap())
                    .add(plotOptionsPaneLayout.createSequentialGroup()
                        .add(plotOptionsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(plotOptionsPaneLayout.createSequentialGroup()
                                .add(25, 25, 25)
                                .add(confLabel)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(confLevelSpinner, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(predIntCheck))
                        .add(77, 77, 77))))
        );
        plotOptionsPaneLayout.setVerticalGroup(
            plotOptionsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(plotOptionsPaneLayout.createSequentialGroup()
                .add(plotOptionsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(regLineCheck)
                    .add(confIntCheck))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(plotOptionsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(loessLineCheck)
                    .add(predIntCheck, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 15, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(plotOptionsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(confLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 17, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(confLevelSpinner, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        plotLabelsPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Plot Labels"));

        plotTitleLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        plotTitleLabel.setLabelFor(plotTitle);
        plotTitleLabel.setText("Plot Title");

        plotTitle.setColumns(12);
        plotTitle.setText("Scatterplot");

        xAxisLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        xAxisLabel.setLabelFor(xAxis);
        xAxisLabel.setText("X-Axis");

        xAxis.setColumns(12);
        xAxis.setText("X");

        yAxisLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        yAxisLabel.setLabelFor(yAxis);
        yAxisLabel.setText("Y-Axis");

        yAxis.setColumns(12);
        yAxis.setText("Y");

        org.jdesktop.layout.GroupLayout plotLabelsPaneLayout = new org.jdesktop.layout.GroupLayout(plotLabelsPane);
        plotLabelsPane.setLayout(plotLabelsPaneLayout);
        plotLabelsPaneLayout.setHorizontalGroup(
            plotLabelsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(plotLabelsPaneLayout.createSequentialGroup()
                .addContainerGap()
                .add(plotLabelsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(xAxisLabel)
                    .add(yAxisLabel)
                    .add(plotTitleLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(plotLabelsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(plotTitle, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 136, Short.MAX_VALUE)
                    .add(xAxis)
                    .add(yAxis, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 136, Short.MAX_VALUE))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        plotLabelsPaneLayout.setVerticalGroup(
            plotLabelsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(plotLabelsPaneLayout.createSequentialGroup()
                .addContainerGap()
                .add(plotLabelsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(plotTitleLabel)
                    .add(plotTitle, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(plotLabelsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(xAxisLabel)
                    .add(xAxis, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(plotLabelsPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(yAxisLabel)
                    .add(yAxis, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(13, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout plotPaneLayout = new org.jdesktop.layout.GroupLayout(plotPane);
        plotPane.setLayout(plotPaneLayout);
        plotPaneLayout.setHorizontalGroup(
            plotPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(plotPaneLayout.createSequentialGroup()
                .addContainerGap()
                .add(plotPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, plotOptionsPane, 0, 427, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, plotPaneLayout.createSequentialGroup()
                        .add(varPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(plotLabelsPane, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        plotPaneLayout.setVerticalGroup(
            plotPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(plotPaneLayout.createSequentialGroup()
                .addContainerGap()
                .add(plotPaneLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(varPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(plotLabelsPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(plotOptionsPane, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-512)/2, (screenSize.height-460)/2, 512, 460);
    }// </editor-fold>//GEN-END:initComponents

    private void regLineCheckActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_regLineCheckActionPerformed
      refreshConfPredInt();
    }//GEN-LAST:event_regLineCheckActionPerformed

    private void loessLineCheckActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loessLineCheckActionPerformed
      refreshConfPredInt();
    }//GEN-LAST:event_loessLineCheckActionPerformed
                         

    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox confIntCheck;
    private javax.swing.JLabel confLabel;
    private javax.swing.JSpinner confLevelSpinner;
    private javax.swing.JCheckBox loessLineCheck;
    private javax.swing.JPanel plotLabelsPane;
    private javax.swing.JPanel plotOptionsPane;
    private javax.swing.JPanel plotPane;
    private javax.swing.JTextField plotTitle;
    private javax.swing.JLabel plotTitleLabel;
    private javax.swing.JCheckBox predIntCheck;
    private javax.swing.JCheckBox regLineCheck;
    private javax.swing.JPanel varPanel;
    private javax.swing.JTextField xAxis;
    private javax.swing.JLabel xAxisLabel;
    private javax.swing.JComboBox xComboBox;
    private javax.swing.JLabel xLabel;
    private javax.swing.JTextField yAxis;
    private javax.swing.JLabel yAxisLabel;
    private javax.swing.JComboBox yComboBox;
    private javax.swing.JLabel yLabel;
    // End of variables declaration//GEN-END:variables

  @Override
  protected void submitButtonAction()
  {
    PreferencesManager prefsManager = new PreferencesManager();
    
    try
    {
      prefsManager.importPreferences();
    }
    catch (Exception ex)
    {
      Logger.getLogger(ConditionalProbability.class.getName()).log(Level.INFO, null, ex);
    }
    
    String cmd = "scatterplot2.JGR("
      + "my.data=" + this.getDatasetPane().getSelectedDataset()
      + ", subset1.name=" + (RUtils.toString(this.getFactorSelectionPane1().getSelectedFactorValues(), ",", "'").length()==0 ? "NULL" : "'" + this.getFactorSelectionPane1().getSelectedFactor() + "'" )
      + ", subset1.val="  + (RUtils.toString(this.getFactorSelectionPane1().getSelectedFactorValues(), ",", "'").length()==0 ? "NULL" : "c(" + RUtils.toString(this.getFactorSelectionPane1().getSelectedFactorValues(), ",", "'") + ")")
      + ", subset2.name=" + (RUtils.toString(this.getFactorSelectionPane1().getSelectedFactorValues(), ",", "'").length()==0 ? "NULL" : "'" + this.getFactorSelectionPane2().getSelectedFactor() + "'" )
      + ", subset2.val="  + (RUtils.toString(this.getFactorSelectionPane2().getSelectedFactorValues(), ",", "'").length()==0 ? "NULL" : "c(" + RUtils.toString(this.getFactorSelectionPane2().getSelectedFactorValues(), ",", "'") + ")")
      + ", x=" + "'" + GUIUtils.getSelectedItem(xComboBox) + "'"
      + ", y=" + "'" + GUIUtils.getSelectedItem(yComboBox) + "'"
      + ", main=" + "'" + plotTitle.getText() + "'"
      + ", xlab=" + "'" + xAxis.getText() + "'"
      + ", ylab=" + "'" + yAxis.getText() + "'"
      + ", add.lm=" + String.valueOf(regLineCheck.isSelected()).toUpperCase()
      + ", add.loess=" + String.valueOf(loessLineCheck.isSelected()).toUpperCase()
      + ", add.conf=" + String.valueOf(confIntCheck.isSelected()).toUpperCase()
      + ", add.pred=" + String.valueOf(predIntCheck.isSelected()).toUpperCase()
      + ", conf=" + ((Double) confLevelSpinner.getValue()).doubleValue() 
      + ", " + prefsManager.getCadstatPreferences().toString()
      + ")";
    JGR.MAINRCONSOLE.execute(cmd, true);
  }

  @Override
  protected void helpButtonAction()
  {
    String cmd = "CADStat.help('scatterplot.JGR')";
    JGR.MAINRCONSOLE.execute(cmd, true);
//    JGR.MAINRCONSOLE.help("Scatterplot");
  }

}
